function aa()
  // abort inreupted program
  abort
endfunction

function cla            
  // clfall    close all figures             
  xdel(winsid())
endfunction

function c=comb(n,k)
  // combination number n!/(k'*(n-k)!)
  // computed in logarithms
  
  num=factL(n);
  den=factL(k)+factL(n-k);
  c=exp(num-den);
endfunction

function c=cvar(x,y)
  // covariance of x,y (second moment)
  n=length(x);
  C=cov(x(:),y(:));
  c=C(1,2)*(n-1)/n;
endfunction

function Lf=factL(n)
  // logarithm of factorial
  Lf=sum(log(1:n));
endfunction

function M=list2mat(L)
  // conversion of list to matrix
  n=length(L);
  for i=1:n
    M(:,i)=L(i)(:);
  end
endfunction

function L=mat2list(M)
  // list from columns of matrix
  n=size(M,2);
  L=list();
  for i=1:n
    L(i)=M(:,i);
  end
endfunction

function j=ranks(a)
  // ranks of a discrete random variable
  // j  ranks, i.e. for a=[3 5 2 2] we get j=[3 4 1.5 1.5]
  //    for equal values in a, we set average rank
  // a  vector of discrete (and maybe also real) values
  [u,i]=gsort(a,'g','i');
  [xx,j]=gsort(i,'g','i');

  av=u(1);
  for i=2:length(a)
    if abs(u(i-1)-u(i))>1e-8
      av=[av u(i)];
    end
  end
  for i=1:length(av)
    ia=find(abs(a-av(i))<1e-8)
    if length(ia)>1
      k=j(ia);
      j(ia)=mean(k);
    end
  end
endfunction

function s=samp(n,a)
  // sampling without repetition
  // n - sample length
  // a - set to be sampled
  m=length(a);
  for i=1:n
    j=fix(m*rand(1,1,'u'))+1;
    s(1,i)=a(j);
    a(j)=[];
    m=length(a);
  end
endfunction

function scatt(x,y)                                                        

  if argn(2)>1, 
    if length(x)~=length(y)
      disp 'The vectors do not have the same length'
      return
    end    
    x=[x(:)';y(:)']; 
  end
  set(scf(),'position',[100 100 800 500]);
  plot(x(1,:),x(2,:),'b.')
endfunction

function T = table(x, y)
  // T=table(x,y)   contingency table T(nx,ny)
  // x,y       data
  
  x = x - min(x) + 1;
  y = y - min(y) + 1;
  nx = length(x);
  ny = length(y);
  kx = max(x);
  ky = max(y);
  nn = min(nx, ny);
  T = zeros(kx, ky);
  for t = 1:nn,
    T(x(t), y(t)) = T(x(t), y(t)) + 1;
  end
endfunction

function [h,f]=vals(a)
  // [h f]=vals(a)  find different values of a variable
  //                and their frequencies               
  // h       values and frequencies  [vals;abs_freq]
  // f       relative frequencies
  
  a=a(:)';
  b=gsort(a,'g','i');
  [v,m]=unique(b);
  dm=diff(m);
  n1=length(b)+1;
  n=[dm n1-m($)];
  f=n/sum(n);
  h=[v(:)';n];

  if sum(n)~=max(size(a))
    disp('Error: in vals.sci')
    return
  end
endfunction

function v=var(x)
  // variance of x ... (1/n)*sum((x-mx)**2)
  n=max(size(x));
  v=variance(x)*(n-1)/n;
endfunction
